# frozen_string_literal: true

source "https://rubygems.org"

gem "standard", "0.2.5"
gem "parallel", "1.19.2" # 1.20+ is required > Ruby 2.4

gem "minitest"
gem "rake"
gem "rake-compiler"
gem "rspec"
gem "test-unit"
